function [Out] = vehEnergy(v_prof,veh_type)

global Crr M g CdA rho Iw rw GB FD eta_gb max_gear EFFnew SPEED TORQUE max_spd a_lim min_spd

WsPerMeter2WhperKmeter = 1/3600*(1/(1/1000));
rad2Rev = 60/(2*pi);
%% Load the vehicle specific data
% The powertrain specific information is loaded; such as the engine map,
% gearbox information, battery data, etc.
load('eng_map.mat');
if max(SPEED(1,:))>4000/rad2Rev
    max_spd = 3100/rad2Rev;
else
    max_spd = max(SPEED(1,:));
end
if min(SPEED(1,:))<=1000/rad2Rev
    min_spd = 1700/rad2Rev;
else
    min_spd = min(SPEED(1,:));
end
a_lim = .15*g;
% Gearbox and final drive information sourced from ANL report: ANL/ESD/12-6
GB_6spd = [(1:6)',[4.15;2.37;1.56;1.16;.86;.52]];
GB_5spd = [(1:5)',[2.56;1.55;1.02;.72;.52]];
GB = GB_5spd;max_gear = max(GB(:,1));
eta_gb = .92; 
FD = 4.7;
%% Run energy analysis for the wheel loads
% Run the analysis over the three driving profiles of interest here.
% Profile analysis
fns = {'hwy','us06','udds'};
for j = 1:length(fns)
    fn = fns{j};    % set the cycle name
    total_time.(fn) = max(v_prof.(fn)(:,1));    % determine the total cycle time
    dt.(fn) = v_prof.(fn)(2,1)-v_prof.(fn)(1,1);    % determine the cycle time step
    Fr.(fn) = Crr*M*g*ones(length(v_prof.(fn)(:,1)),1); % compute the rolling losses
    Fd.(fn) = CdA*rho*0.5*v_prof.(fn)(:,2).^2; % compute the drag losses
    a.(fn) = zeros(size(Fr.(fn)));a.(fn)(1)=0;a.(fn)(length(Fr.(fn)))=(v_prof.(fn)(length(Fr.(fn)),2)-v_prof.(fn)(length(Fr.(fn))-1,2))/dt.(fn); % compute the first and last accelerations
    Finr.(fn) = zeros(size(Fr.(fn)));Finr.(fn)(length(Fr.(fn)),1)=(M+4*(Iw/rw^2))*a.(fn)(length(Fr.(fn)));  % compute the first and last inertial forces
    % Computes the accelerations and inertial forces usign a central
    % difference approximation, as opposed to only a backward
    % difference.  The first and last accel's and Finr are computed
    % using a backward difference.
    for i = 2:length(v_prof.(fn)(:,1))-1
        a.(fn)(i) = ((v_prof.(fn)(i+1,2)-v_prof.(fn)(i,2))/dt.(fn) + (v_prof.(fn)(i,2)-v_prof.(fn)(i-1,2))/dt.(fn))/2; % central difference acceleration calculation
        Finr.(fn)(i) = (M+4*(Iw/rw^2))*a.(fn)(i);
    end
    Ftr.(fn) = Fr.(fn)+Fd.(fn)+Finr.(fn);
    % Initialize the needed variables
    brake_energy.(fn)=0;pro_energy.(fn)=0;brake_dist.(fn)=0;pro_dist.(fn)=0;
    net_power.(fn) = zeros(size(Fr.(fn)));pro_power.(fn) = zeros(size(Fr.(fn)));
    brake_power.(fn) = zeros(size(Fr.(fn)));net_energy.(fn) = 0;dist.(fn)=0;
    t_idle.(fn) = 0;
    % Computes the powers and energies of interest
    for i = 1:length(Fr.(fn))
        net_power.(fn)(i) = Ftr.(fn)(i)*v_prof.(fn)(i,2);
        net_energy.(fn) = net_energy.(fn) + net_power.(fn)(i)*dt.(fn);
        dist.(fn) = dist.(fn) + v_prof.(fn)(i,2)*dt.(fn);
        if net_power.(fn)(i) > 0 % if the vehicle is is drawing positive power (propulsion) 
            pro_power.(fn)(i) = net_power.(fn)(i);                          % sets the propulsion power
            pro_energy.(fn) = pro_energy.(fn)+pro_power.(fn)(i)*dt.(fn);    % performs the propulsion energy integration
            pro_dist.(fn) = pro_dist.(fn) + v_prof.(fn)(i,2)*dt.(fn);       % performs the propulsion distance integration   
        end     
        if net_power.(fn)(i) < 0 % if the vehicle is drawing negative power (braking)
            brake_power.(fn)(i) = net_power.(fn)(i);                                % sets the brake power
            brake_energy.(fn) = brake_energy.(fn) + brake_power.(fn)(i)*dt.(fn);    % performs the braking energy integration
            brake_dist.(fn) = brake_dist.(fn) + v_prof.(fn)(i,2)*dt.(fn);           % performs the braking distance integration
        end  
        if v_prof.(fn)(i,2) == 0
            t_idle.(fn) = t_idle.(fn)+dt.(fn);
        end
    end
    % Compute the economies, peaks, and averages of interest
    wheels.pro_economy.(fn) = pro_energy.(fn)/pro_dist.(fn)*WsPerMeter2WhperKmeter;
    wheels.brake_economy.(fn) = abs(brake_energy.(fn))/brake_dist.(fn)*WsPerMeter2WhperKmeter;
    wheels.net_economy.(fn) = net_energy.(fn)/dist.(fn)*WsPerMeter2WhperKmeter;
    wheels.avg_pro_power.(fn) = mean(pro_power.(fn)(pro_power.(fn)~=0))/1000;
    wheels.peak_pro_power.(fn) = max(pro_power.(fn))/1000;
    wheels.peak_force.(fn) = max(abs(Ftr.(fn)))/1000;
    wheels.percent_idle_time.(fn) = t_idle.(fn)/total_time.(fn)*100;
    wheels.a.(fn) = a.(fn);
end
Out.wheels = wheels;

%% Run the analysis for the conventional vehicle
% Of importance here is to determine gear position. A function, called
% "decideShift" performs the shift control.  The engine speed and load
% requirements are taken into account for shifting.
if strcmp(veh_type,'conv')
    % run for all speed profiles
    for j = 1:length(fns)
        fn = fns{j};
        gear = ones(length(v_prof.(fn)(:,1)),1);
        eng_spd.(fn) = zeros(length(v_prof.(fn)(:,1)),1);
        eng_eff.(fn) = ones(length(v_prof.(fn)(:,1)),1);
        Out.conv.net_power.(fn) = zeros(length(v_prof.(fn)(:,1)),1);
        eng_power.(fn) = zeros(length(v_prof.(fn)(:,1)),1);
        fuel_power.(fn) = zeros(length(v_prof.(fn)(:,1)),1);
        % run for the full time of the profile
        for i = 1:length(v_prof.(fn)(:,1))
            if net_power.(fn)(i)>0
                eng_spd.(fn)(i,1) = v_prof.(fn)(i,2)*GB(gear(i),2)*FD/rw; % rad/s
                eng_trq.(fn)(i,1) = Ftr.(fn)(i)/GB(gear(i),2)/FD*rw*1/eta_gb; % N-m
                % Find the engine efficiency at the current operating point
                [val,ind_spd.(fn)(i,1)]=min(abs(eng_spd.(fn)(i)-SPEED(1,:)));
                [val,ind_trq.(fn)(i,1)]=min(abs(eng_trq.(fn)(i)-TORQUE(:,1)));
                eng_eff.(fn)(i) = EFFnew(ind_trq.(fn)(i),ind_spd.(fn)(i)); % interpolate on 2d engine map
                if isnan(eng_eff.(fn)(i))
                    eng_eff.(fn)(i)=max(EFFnew(:,ind_spd.(fn)(i)));
                end
                Out.conv.net_power.(fn)(i,1) = eng_spd.(fn)(i)*eng_trq.(fn)(i);
            else
                eng_spd.(fn)(i,1) = v_prof.(fn)(i,2)*GB(gear(i),2)*FD/rw; % rad/s
                eng_trq.(fn)(i,1) = 0; % N-m
                eng_eff.(fn)(i,1) = 1;
                Out.conv.net_power.(fn)(i,1) = brake_power.(fn)(i);
            end
            eng_power.(fn)(i,1) = eng_spd.(fn)(i)*eng_trq.(fn)(i);
            fuel_power.(fn)(i,1) = eng_power.(fn)(i)/(eng_eff.(fn)(i)/100);

            gear(i+1) = decideShift(gear(i),eng_spd.(fn)(i),a.(fn)(i));
        end
        Out.conv.eng_energy.(fn) = trapz(v_prof.(fn)(:,1),eng_power.(fn));
        Out.conv.fuel_energy.(fn) = trapz(v_prof.(fn)(:,1),fuel_power.(fn));
        Out.conv.net_energy.(fn) = trapz(v_prof.(fn)(:,1),Out.conv.net_power.(fn));
        % Output the gear, eng_spd, and veh_spd. These are indexed over time for each of the drive schedules used here. 
        Out.conv.gear.(fn) = gear(1:length(gear)-1); 
        Out.conv.eng_spd.(fn) = eng_spd.(fn)*rad2Rev;
        Out.conv.veh_spd.(fn) = v_prof.(fn);
        Out.conv.eng_eff.(fn) = eng_eff.(fn);
        Out.conv.eng_trq.(fn) = eng_trq.(fn);
        Out.conv.ind_spd.(fn) = ind_spd.(fn);
        Out.conv.ind_trq.(fn) = ind_trq.(fn);
                
        % Compute the economies, peaks, and averages of interest
        Out.conv.fuel_economy.(fn) = Out.conv.fuel_energy.(fn)/pro_dist.(fn)*WsPerMeter2WhperKmeter;
        Out.conv.fuel_mileage.(fn) = dist.(fn)/(Out.conv.fuel_energy.(fn)/33333);
        Out.conv.net_economy.(fn) = Out.conv.net_energy.(fn)/dist.(fn)*WsPerMeter2WhperKmeter;
        only_eng_power.(fn) = eng_power.(fn)(eng_power.(fn)~=0);
        Out.conv.avg_pro_power.(fn) = mean(only_eng_power.(fn))/1000;
        Out.conv.peak_pro_power.(fn) = max(eng_power.(fn))/1000;
    end
end
%% Run the analysis for the electric vehicle

% Need to add GB_ev, FD_ev, mot_SPEED, mot_TORQUE, EFF_ev
% EFF_ev needs to include a battery and motor model.

% Of importance here is to determine gear position. A function, called
% "decideShift" performs the shift control.  The engine speed and load
% requirements are taken into account for shifting.
if strcmp(veh_type,'ev')
    % run for all speed profiles
    for j = 1:length(fns)
        fn = fns{j};
        % run for the full time of the profile
        for i = 1:length(v_prof.(fn)(:,1))-1
            mot_spd.(fn)(i) = v_prof.(fn)(i,2)*GB_ev*FD_ev/rw; % rad/s
            mot_trq.(fn)(i) = Ftr.(fn)(i)*GB_ev*FD_ev/rw*1/eta_gb; % N-m
            % Find the powertrain (batt+motor) efficiency at the current operating point
            [val,ind_spd]=min(mot_spd.(fn)(i)-mot_SPEED(1,:));
            [val,ind_trq]=min(mot_trq.(fn)(i)-mot_TORQUE(:,1));
            evPT_eff.(fn)(i) = EFFev(ind_trq,ind_spd); % interpolate on 2d engine map
        end
       % Output the gear, eng_spd, and veh_spd. These are indexed over time for each of the drive schedules used here. 
       Out.conv.mot_spd.(fn) = mot_spd.(fn)*rad2Rev;
       Out.conv.veh_spd.(fn) = v_prof.(fn);
    end
end