%% This function performs the performance analysis, which consists of the average 0-60 mph acceleration power and the average power consumption for grades.
% The function performs the analysis based on the vehicle type, which may
% be either wheels,conv,ev,or shev.  Currently the code only operates for
% the wheels analysis.
function [outs] = vehPerf(veh_type)

global Crr M g CdA rho Iw rw MPH2mps Mgvwr

%% 0-60 Analysis
% Average analysis, not a true dynamic analysis using an engine map
tf = 11; %[s], this is the minimum requirement for 0-60 acceleration time set by EcoCAR3
vf = 60*MPH2mps;
ti = 0; vi = 0;
dv = vf - vi;dt = tf - ti;
a_avg = dv/dt;
% Compute the linear velocit profile using the constant acceleration
% assumption.
t = (0:.1:tf)';
vel_prof = a_avg*t;
% Compute the forces over the acceleration.
Fd_zts = CdA/2*rho*vel_prof.^2;
Fr_zts = Crr*M*g*ones(size(Fd_zts));
Finr_zts = (M+4*Iw/rw^2)*a_avg*ones(size(Fd_zts));
Ftr_zts = Fd_zts+Fr_zts+Finr_zts;
power_zts = Ftr_zts.*vel_prof;
avg_power_zts = mean(power_zts);
% Create the output structure
outs.wheel.avg_power_zts = avg_power_zts/1000;
%% Grade analysis
% computed for a climb of 3.5% at 60 mph with the mass set to the GVWR
alpha = atan(3.5/100);
% Update the output structure with the grade power consumption
outs.wheel.avg_power_grade = (Crr*Mgvwr*g + CdA/2*rho*vf^2 + Mgvwr*g*sin(alpha))*vf/1000;